#!/usr/bin/env python
"""
	This software was developed by the University of Tennessee as part of the
	Distributed Data Analysis of Neutron Scattering Experiments (DANSE)
	project funded by the US National Science Foundation.

	If you use DANSE applications to do scientific research that leads to
	publication, we ask that you acknowledge the use of the software with the
	following sentence:

	"This work benefited from DANSE software developed under NSF award DMR-0520547."

	copyright 2008, University of Tennessee
"""

""" Provide functionality for a C extension model

	WARNING: THIS FILE WAS GENERATED BY WRAPPERGENERATOR.PY
 	         DO NOT MODIFY THIS FILE, MODIFY ..\c_extensions\SquareWell.h
 	         AND RE-RUN THE GENERATOR SCRIPT

"""

from sans.models.BaseComponent import BaseComponent
from sans_extension.c_models import CSquareWellStructure
import copy    
    
class SquareWellStructure(CSquareWellStructure, BaseComponent):
    """ Class that evaluates a SquareWellStructure model. 
        This file was auto-generated from ..\c_extensions\SquareWell.h.
        Refer to that file and the structure it contains
        for details of the model.
        List of default parameters:
         effect_radius   = 50.0 [A]
         volfraction     = 0.04 
         welldepth       = 1.5 [kT]
         wellwidth       = 1.2 

    """
        
    def __init__(self):
        """ Initialization """
        
        # Initialize BaseComponent first, then sphere
        BaseComponent.__init__(self)
        CSquareWellStructure.__init__(self)
        
        ## Name of the model
        self.name = "SquareWellStructure"
        ## Model description
        self.description =""" Structure Factor for interacting particles:             .
		
		The interaction potential is
		
		U(r)= inf   , r < 2R
		= -d    , 2R <= r <=2Rw
		= 0     , r >= 2Rw
		
		R: effective radius (A)of the particle
		v: volume fraction
		d: well depth
		w: well width; multiples of the
		particle diameter
		
		Ref: Sharma, R. V.; Sharma,
		K. C., Physica, 1977, 89A, 213."""
       
        ## Parameter details [units, min, max]
        self.details = {}
        self.details['effect_radius'] = ['[A]', None, None]
        self.details['volfraction'] = ['', None, None]
        self.details['welldepth'] = ['[kT]', None, None]
        self.details['wellwidth'] = ['', None, None]

        ## fittable parameters
        self.fixed=['effect_radius.width']
        
        ## parameters with orientation
        self.orientation_params =[]
   
    def clone(self):
        """ Return a identical copy of self """
        return self._clone(SquareWellStructure())   
        
    def __getstate__(self):
        """ return object state for pickling and copying """
        model_state = {'params': self.params, 'dispersion': self.dispersion, 'log': self.log}
        
        return self.__dict__, model_state
        
    def __setstate__(self, state):
        """ create object from pickled state """
        
        self.__dict__, model_state = state
        self.params = model_state['params']
        self.dispersion = model_state['dispersion']
        self.log = model_state['log']
       	
   
    def run(self, x = 0.0):
        """ Evaluate the model
            @param x: input q, or [q,phi]
            @return: scattering function P(q)
        """
        
        return CSquareWellStructure.run(self, x)
   
    def runXY(self, x = 0.0):
        """ Evaluate the model in cartesian coordinates
            @param x: input q, or [qx, qy]
            @return: scattering function P(q)
        """
        
        return CSquareWellStructure.runXY(self, x)
        
    def evalDistribition(self, x = []):
        """ Evaluate the model in cartesian coordinates
            @param x: input q[], or [qx[], qy[]]
            @return: scattering function P(q[])
        """
        return CSquareWellStructure.evalDistribition(self, x)
        
    def calculate_ER(self):
        """ Calculate the effective radius for P(q)*S(q)
            @return: the value of the effective radius
        """       
        return CSquareWellStructure.calculate_ER(self)
        
    def set_dispersion(self, parameter, dispersion):
        """
            Set the dispersion object for a model parameter
            @param parameter: name of the parameter [string]
            @dispersion: dispersion object of type DispersionModel
        """
        return CSquareWellStructure.set_dispersion(self, parameter, dispersion.cdisp)
        
   
# End of file
